@@ -1,16 +1,23 @@
+from io import StringIO
+from pathlib import Path
+# pip install rich
+from rich.console import Console
 from datetime import datetime
 from enum import Enum
-from inspect import stack, getframeinfo
+import hashlib
+from inspect import stack
 from os import path
+import traceback
 from typing import Final, Literal
 from typing import Optional, Any, Callable, Union
 
+from .log_row import LogRow
 from .helpful import MetaLogger
 from .independent.helpful import toBitSize
 from .independent.log_file import LogFile
 from .independent.zip_file import ZippFile, ZipCompression
 
-__all__ = ["logger", "loglevel"]
+__all__ = ["logger", "loglevel", ]
 
 
 class CompressionLog(Enum):
@@ -56,14 +63,13 @@ class loglevel:
             self,
             title_logger: str,
             int_level: int = 10,
-            fileout: Optional[str] = None,
+            fileout: Optional[Path] = None,
             console_out: bool = True,
             color_flag: str = "",
             color_title_logger: str = "",
             max_size_file: Optional[Union[int, str]] = "10mb",
             max_len_console: int = 0,
             compression: Optional[Union[CompressionLog, Callable]] = None,
-            template_file: str = "[{date_now}][{title_logger}][{flags}]:{data}\n",
             template_console: str = "{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}:{data}",
             **kwargs,
     ):
@@ -79,13 +85,12 @@ class loglevel:
             - kb - Например 10kb
             - mb - Например 1mb
             - None - Без ограничений
-        :param max_len_console: Скольки максимум символов выводить в консоль, остальное обрезать.По умолчанию не обрезать  
+        :param max_len_console: Скольки максимум символов выводить в консоль, остальное обрезать.По умолчанию не обрезать
         :param compression: Что делать с файлам после достижение ``max_size_file``
-        :param template_file: Доступные аргументы в :meth:`allowed_template_loglevel`
         :param template_console: Доступные аргументы в :meth:`allowed_template_loglevel`
         """
         self.title_logger: str = title_logger
-        self.fileout: Optional[str] = fileout
+        self.fileout: Optional[Path] = fileout
         if fileout:
             # Если указан файл, то добавляем функцию для записи в файл
             self._base_logic = lambda data, flags: (self._file_write(
@@ -102,7 +107,6 @@ class loglevel:
         self.max_len_console: int = max_len_console
         self.compression: Callable = compression if compression else CompressionLog.rewrite_file
         self.int_level: int = int_level
-        self.template_file: str = template_file
         self.template_console: str = template_console
 
         #: Сколько раз было записей в лог файл, до выполнения
@@ -130,7 +134,7 @@ class loglevel:
         """
         # Формируем сообщение в файл
         log_formatted = allowed_template_loglevel(
-            self.template_file, data, flags, self, 'file')
+            None, data, flags, self, 'file')
         # Записываем в файл
         _f = LogFile(self.fileout)
         _f.appendFile(log_formatted)
@@ -185,7 +189,7 @@ class loglevel:
         """
         Вернуть обновленную версию loglevel
 
-        kwargs: Обновление для loglevel.__init__ 
+        kwargs: Обновление для loglevel.__init__
         """
         return loglevel(**dict(self.__dict__, **kwargs))
 
@@ -200,7 +204,7 @@ class loglevel_extend(loglevel):
     def _file_write(self, data: Any, flags: str):
         # Формируем сообщение в файл
         log_formatted = allowed_template_loglevel.debug_new(
-            self.template_file, data, flags, self, 'file')
+            None, data, flags, self, 'file')
         # Записываем в файл
         _f = LogFile(self.fileout)
         _f.appendFile(log_formatted)
@@ -218,28 +222,98 @@ class loglevel_extend(loglevel):
         """
         Вернуть обновленную версию loglevel_extend
 
-        kwargs: Обновление для loglevel_extend.__init__ 
+        kwargs: Обновление для loglevel_extend.__init__
         """
         return loglevel_extend(**dict(self.__dict__, **kwargs))
 
 
+class loglevel_form_error(loglevel_extend):
+    """
+    Логгер для ошибок
+    """
+
+    def _file_write(self, data: Any, flags: str):
+        # Формируем сообщение в файл
+        log_formatted, error = self.detail(data)
+        # Записываем в файл
+        _f = LogFile(self.fileout)
+        _f.appendFile(log_formatted)
+        # Проверить размер файла, если размер больше ``self.max_size_file`` то произойдет ``self.compression``
+        self._check_size_log_file(_f)
+        return error
+
+    def __call__(self, data: str, flags: list[str] = ""):
+        """
+        Вызвать логгер
+
+        :param data:
+        :param flags:
+        """
+        # Если уровень доступа выше или равен требуемому
+        if self.int_level >= self.required_level:
+            # Выполняем логику логера
+            return self._base_logic(data, flags)[0]
+
+    def _console_print(self, data: Any, flags: str):
+        if self.console_out:
+            # Формируем сообщение в консоль
+            log_formatted = allowed_template_loglevel.debug_new(
+                self.template_console, data, flags, self, 'console')
+            error: str = traceback.format_exc()
+            id_error_log: str = hashlib.md5(error.encode("utf-8")).hexdigest()
+            log_formatted += f":::::ERROR_LOG:::::: {id_error_log}"
+            print(log_formatted)
+
+    def detail(self, msg: str) -> str:
+        """Подробный вывод ошибки
+
+        :param path_log: Путь лог файлу
+        :param msg: Сообщение
+        :return: Текст ошибки, и идентификатор(ERROR_LOG) на полный стек ошибки в файле
+        """
+        # traceback.format_exception(e) # В виде списка
+        # traceback.format_exc(e) # В виде текста
+        error: str = traceback.format_exc()
+        id_error_log: str = hashlib.md5(error.encode("utf-8")).hexdigest()
+        time_now = datetime.now()
+        #
+        console = Console(width=180, file=StringIO())
+        console.print_exception(show_locals=True)
+        str_output = console.file.getvalue()
+        #
+        res = """{0}{1}{2}""".format(
+            f'BEGIN---|{time_now}|{id_error_log}|{msg}\n',
+            str_output,
+            f'END__---|{time_now}|{id_error_log}|{msg}\n'
+        )
+        return res, f'{error}\n:::::ERROR_LOG::::: {id_error_log}'
+
+    def updateCopy(self, **kwargs):
+        """
+        Вернуть обновленную версию loglevel_form_error
+
+        kwargs: Обновление для loglevel_form_error.__init__
+        """
+        return loglevel_form_error(**dict(self.__dict__, **kwargs))
+
+
 class allowed_template_loglevel:
     """
     Доступные ключи для шаблона лог сообщения в файл
 
     :Пример передачи:
 
-    ``{level}{flags}{data}\n``
-    ``{color_loglevel}{level}{reset}{color_flag}{flags}{reset}``
+    `{level}{flags}{data}`
+    `{color_loglevel}{level}{reset}{color_flag}{flags}{reset}`
     """
 
     def __new__(
-            cls,
-            _template: str,
-            data,
-            flags,
-            root_loglevel: loglevel,
-            type_d: Literal['file', 'console'] = 'console'
+        cls,
+        _template: str,
+        data,
+        flags,
+        root_loglevel: loglevel,
+        type_d: Literal['file', 'console'] = 'console'
     ) -> str:
         """
         Обычное сообщение
@@ -248,31 +322,34 @@ class allowed_template_loglevel:
         :param flags:
         :param data:
         """
+        if type_d == 'console':
+            return _template.format(
+                #  Название логера
+                title_logger=root_loglevel.title_logger,
+                # Флаги
+                flags=';'.join([str(x) for x in flags]),
+                # Если нужно обрезаем сообщение
+                data=data[:root_loglevel.max_len_console] if root_loglevel.max_len_console > 0 else data,
+                #  Дата создания сообщения
+                date_now=datetime.now(),
+                # Закрыть цвет
+                reset=MetaLogger.reset_,
+                #  Цвет заголовка логера
+                color_title_logger=root_loglevel.color_title_logger,
+                # Цвет флага
+                color_flag=root_loglevel.color_flag,
+            )
+        else:
+            # file
+            return LogRow._(title=root_loglevel.title_logger, flags=flags, data=data, is_trace=False, stack_back=5)
 
-        return _template.format(
-            #  Название логера
-            title_logger=root_loglevel.title_logger,
-            # Флаги
-            flags=';'.join([str(x) for x in flags]),
-            # Если нужно обрезаем сообщение
-            data=data[:root_loglevel.max_len_console] if type_d == 'console' and root_loglevel.max_len_console > 0 else data,
-            #  Дата создания сообщения
-            date_now=datetime.now(),
-            # Закрыть цвет
-            reset=MetaLogger.reset_,
-            #  Цвет заголовка логера
-            color_title_logger=root_loglevel.color_title_logger,
-            # Цвет флага
-            color_flag=root_loglevel.color_flag,
-        )
-
-    @classmethod
+    @ classmethod
     def debug_new(
-            cls,
-            _template: str,
-            data,
-            flags,
-            root_loglevel: loglevel,
+        cls,
+        _template: str,
+        data,
+        flags,
+        root_loglevel: loglevel,
         type_d: Literal['file', 'console'] = 'console'
     ):
         """
@@ -283,34 +360,38 @@ class allowed_template_loglevel:
         :param flags:
         :param data:
         """
-
-        caller = getframeinfo(stack()[4][0])
-        return _template.format(
-            #  Название логера
-            title_logger=root_loglevel.title_logger,
-            # Флаги
-            flags=';'.join([str(x) for x in flags]),
-            # Если нужно обрезаем сообщение
-            data=data[:root_loglevel.max_len_console] if type_d == 'console' and root_loglevel.max_len_console > 0 else data,
-            #  Дата создания сообщения
-            date_now=datetime.now(),
-            # Закрыть цвет
-            reset=MetaLogger.reset_,
-            #  Цвет заголовка логера
-            color_title_logger=root_loglevel.color_title_logger,
-            # Цвет флага
-            color_flag=root_loglevel.color_flag,
-            # Номер строки
-            line_call=caller.lineno,
-            # Функция в которой вызвана функция
-            func_call=caller.function,
-            # Контекст
-            context_call=''.join(caller.code_context[0].split()),
-            # Абсолютный путь к файлу в котором вызвана функция
-            abs_file_call=caller.filename,
-            # Файл в котором вызвана функция
-            file_call=path.basename(caller.filename)
-        )
+        if type_d == 'console':
+            caller = stack()[4]
+            return _template.format(
+                #  Название логера
+                title_logger=root_loglevel.title_logger,
+                # Флаги
+                flags=';'.join([str(x) for x in flags]),
+                # Если нужно обрезаем сообщение
+                data=str(data)[
+                    :root_loglevel.max_len_console] if root_loglevel.max_len_console > 0 else data,
+                #  Дата создания сообщения
+                date_now=datetime.now(),
+                # Закрыть цвет
+                reset=MetaLogger.reset_,
+                #  Цвет заголовка логера
+                color_title_logger=root_loglevel.color_title_logger,
+                # Цвет флага
+                color_flag=root_loglevel.color_flag,
+                # Номер строки
+                line_call=caller.lineno,
+                # Функция в которой вызвана функция
+                func_call=caller.function,
+                # Контекст
+                context_call=''.join(caller.code_context[0].split()),
+                # Абсолютный путь к файлу в котором вызвана функция
+                abs_file_call=caller.filename,
+                # Файл в котором вызвана функция
+                file_call=path.basename(caller.filename)
+            )
+        else:
+            # file
+            return LogRow._(title=root_loglevel.title_logger, flags=flags, data=data, is_trace=True, stack_back=5)
 
 
 class logger:
@@ -346,19 +427,26 @@ class logger:
     error = loglevel_extend(
         "ERROR",
         int_level=40,
-        template_console="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
-        template_file="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
+        template_console="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}:{line_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
+        color_title_logger=MetaLogger.read,
+        color_flag=MetaLogger.yellow,
+    )
+    errordet = loglevel_form_error(
+        "ERROR",
+        int_level=40,
+        fileout='./errordet.log',
+        template_console="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}:{line_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
         color_title_logger=MetaLogger.read,
         color_flag=MetaLogger.yellow,
     )
     warning = loglevel_extend(
         "WARNING",
         int_level=30,
-        template_console="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
-        template_file="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
+        template_console="{color_title_logger}[{title_logger}]{reset}{color_flag}[{flags}]{reset}{color_flag}[{date_now}][{file_call}:{func_call}:{line_call}]{reset}\n{color_flag}Text:{reset}\t{data}\n{color_flag}Path:{reset}\t{abs_file_call}{line_call}\n{color_flag}Context:{reset}\t{context_call}{color_title_logger}\n[/END_{title_logger}]{reset}",
         color_flag=MetaLogger.read,
         color_title_logger=MetaLogger.yellow,
     )
+
     #: Логгер для системных задач
     system_info: Final[loglevel] = loglevel(
         "SYSTEM",
